/* ul-warning
 * copyright 2009 Bobby Powers, released under the GPLv2+
 * see COPYING for details
 *
 * starts our graphical boot, but only if the DCON is frozen
 */
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <linux/vt.h>
#include <linux/kd.h>

#include "utils.h"

/* ul-image in 565 format */
static const char *UL_IMAGE = "/usr/share/boot-anim/ul_warning.565";

static void
initial_setup ()
{
  int vt, console;

  /* step 1 - disable cursor on VT2 */
  vt = open ("/dev/tty2", O_WRONLY);
  if (vt < 0)
  {
    write (1, "Couldn't open /dev/tty2 to set mode.\n", 37);
    return;
  }

  /* steal this console for our graphics */
  ioctl (vt, KDSETMODE, KD_GRAPHICS);
  close (vt);

  /* change to vt2 */
  console = open ("/dev/console", O_RDWR);
  if (console < 0)
  {
    write (1, "Couldn't open /dev/console to switch VTs.\n", 42);
    return;
  }
  chvt_in_userspace (console, 2);
  close (console);

  /* redirect console messages to vt1, even though cscott's notes
   * indicate this doesn't work. */
  vt = open ("/dev/tty1", O_RDWR);
  if (console < 0)
  {
    write (1, "Couldn't open /dev/tty1 for redirection.\n", 41);
    return;
  }
  ioctl (vt, TIOCCONS);
  close (vt);
}

int
main (int argc, char *argv[])
{
  int fb_file = 0;
  unsigned short *fb = NULL;

  fb_file = open ("/dev/fb0", O_RDWR);
  if (fb_file < 0)
  {
    write (1, "Couldn't open /dev/fb0 for boot animation.\n", 44);
    return 1;
  }

  fb = (unsigned short *)mmap (NULL,
                               FB_SIZE_B + EXTRA_SPACE,
                               PROT_READ|PROT_WRITE,
                               MAP_SHARED,
                               fb_file,
                               0);

  /* redirect console output to tty1, and set terminal to graphics mode */
  initial_setup ();

  /* write the default image, which we then will paint image diffs over. */
  write_image (fb, UL_IMAGE);

  dcon_freeze (1);

  munmap (fb, FB_SIZE_B);
  close (fb_file);

  return 0;
}

